/*
 * Copyright (c) 2017 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *                                           Anthony Mallet on Tue Feb 28 2017
 */
#include "acarduio.h"

#include <stdio.h>

#include "arduio_c_types.h"
#include "codels.h"


/* --- Task main -------------------------------------------------------- */


/** Codel ario_main_init of task main.
 *
 * Triggered by arduio_start.
 * Yields to arduio_main.
 */
genom_event
ario_main_init(arduio_ids *ids, genom_context self)
{
  return arduio_main;
}


/** Codel ario_main_perm of task main.
 *
 * Triggered by arduio_main.
 * Yields to arduio_pause_main.
 */
genom_event
ario_main_perm(const arduio_conn_s *conn, genom_context self)
{
  /* request state */
  if (conn && conn->chan.fd >= 0)
    ario_send_msg(&conn->chan, "s");

  return arduio_pause_main;
}


/* --- Activity set_dir ------------------------------------------------- */

/** Codel ario_set_dir of activity set_dir.
 *
 * Triggered by arduio_start.
 * Yields to arduio_pause_ack.
 * Throws arduio_e_sys, arduio_e_baddir.
 */
genom_event
ario_set_dir(uint8_t n, arduio_dir_s dir, const arduio_conn_s *conn,
             genom_context self)
{
  int s;
  uint8_t d;
  if (!conn) {
    arduio_e_baddev_detail d;
    snprintf(d.dev, sizeof(d.dev), "not connected to device");
    return arduio_e_baddev(&d, self);
  }

  switch(dir) {
    case arduio_IO_IN:		d = 0; break;
    case arduio_IO_IN_PU:	d = 1; break;
    case arduio_IO_OUT:		d = 2; break;
    case arduio_IO_PWM:		d = 3; break;
  }
  s = ario_send_msg(&conn->chan, "d%1%1", n, d);
  if (s) return ario_e_sys_error(NULL, self);

  return arduio_pause_ack;
}

/** Codel ario_set_dir_ack of activity set_dir.
 *
 * Triggered by arduio_ack.
 * Yields to arduio_ether.
 * Throws arduio_e_sys, arduio_e_baddir.
 */
genom_event
ario_set_dir_ack(uint8_t n, arduio_dir_s dir, const arduio_ios *ios,
                 genom_context self)
{
  arduio_state_s *data = ios->data(self);

  if (n >= data->diodir._length) return arduio_e_baddir(self);
  if (data->diodir._buffer[n] != dir) return arduio_e_baddir(self);

  return arduio_ether;
}
